; From Jason Cook:
;
; For updates: https://hub.inktada.com/channel/rom1diskmagic
;
; This is a ROM 1 patch to support disk (IEEE-488) compatibility with only 
; burning and changing a single ROM ($F000-$F7FF) H4 in an original ROM 1 
; Commodore PET.
;
;-------------------------------------------------------------------------------
;
; This code's concept was derived from the original IEEE-488 ROM 1 patch 
; written by André Fachat and ROM 1 disassembly. Vossi then created a 
; single ROM replacement for $F000-F7FF and this is an enhancement of
; that concept.
;
; Changes and Improvements: Only 4 error messages shortened rather than 7.
; Patch code simplified due to larger continuous space availability.
;
; My shortened error messages are:
;
; # = too many files (never called by any ROM code, as far as I can see)
; OPEN = file open
; FNOPN = file not open
; DEVNP = device not present
;
; Vossi's original shortened error message reduction were:
;
; TMF = too many files (never called by any ROM, as far as I can see)
; OPEN = file open
; FNF = file not found
; DEVNP = device not present
; NIF = not input file
; NOF = not output file
; T = on tape #
;
; In order to wrap my head around what was included in the ROM 1 patch and feel
; more comfortable with what exactly the patch did, I disassembled
; the binaries. I then added comments from ROM 1 disassembly
; found here: http://www.zimmers.net/anonftp/pub/cbm/src/pet/rom-1.html
;
;-------------------------------------------------------------------------------
;
; This file is purposely self contained with no dependencies, as it needs to
; live by itself. Therefore, all memory addresses are self contained.
;
; To make it easy to find the patch points and sanity check the code,
; each area surrounded with four lines of ;;;;; mark a patch point.
; All other code, including exact locations, are identical to the
; original ROM1 code.
;
; The original patch written by André Fachat is quite impressive as he
; kept memory matching exact, but used memory areas between $EF00-$EFFF for
; the patches. Vossi created a nice solution by putting André's patch code
; between shortened error messages, while keeping the original pointers to the
; error messages exactly the same.
;
; Like André's version, LOAD and VERIFY mostly returns OK. You must
; review the ST (status) basic variable to get the result or read location
; $20C. AND #$40 (bit 6) indicates end of file reached, which is a good
; indicator success. You may also want to check any of the bits 4, 5, 7 set, 
; as any set would indicate error.
; Quick test BASIC:
;   IF ((ST) AND 64)>0 THEN REM SUCCESS CODE HERE
; Quick test assembler:
;   LDA $20C
;   AND #$40 ;(bit 6) indicates end of file reached, NON ZERO means success.
;
; Save is less reliable, but ST=0 is the best indicator of success. Review
; status bits for PET.
;
; I was able to reclaim the unused code that André's patch orphaned
; at iF71C.
;
; Of the four error messages reduced, I can't find where the ROM calls
; "Too many files" (bug due to rush of release for ROM 1 in the 70s), so there
; are only three shortened error messages that an end user would see.
;
; LDY #$ instruction modified for text output:
; file_open_error was #$0e, now #$13
;
; The patched ROM code stays byte aligned with the original ROM code.
;
; While I have tested, the code could use another pair of eyes to make sure
; nothing looks or behaves different from André's original patch. 
;
; Assembler used: ca65 https://cc65.github.io/doc/cl65.html :
; cl65 -o THISFILENAME.bin -t none -l THISFILENAME.lst THISFILENAME.s
;
;-------------------------------------------------------------------------------

	.org	$f000
x08 =			$08
IINDEX =		$71
VARSTART =	$7C
iC8 =			$C8
xCA =			$CA
xE2 =			$E2
FNLEN =		$EE
SADDR =		$E3
EADDR =		$E5
xE9 =			$E9
LFN =			$EF	;Logical File Number
SA =			$F0	;Current Secondary Address
FA	=			$F1	;Current Device Number
SECADDRHI =	$F2
TAPEBUFFER = $F3
LINENO =		$F5
LADDR =		$F7
OSPTR =		$F9

STACK =		$0100
TCLOCK =		$0200
TIMER1 =		$0205
KEYMSK =		$0209
VERFL =		$020B
STATUS =		$020C	;Kernal I/O Status Word: ST
NKEYS =		$020D
OUTFL =		$021D
x021E =		$021E
x0220 =		$0220
x0221	=		$0221

OUTBUF =		$0222
LFTAB =		$0242
FATAB =		$024C
SATAB =		$0256
x0260	=		$0260
x0261 =		$0261
NFILES =		$0262
CURIDEV =	$0263
CURODEV =	$0264
x0268 =		$0268
x026A =		$026A
x0270 =		$0270
SECDATA =	$0279

iC379 =		$C379
iC430 =		$C430
iC59A =		$C59A
iC581 =		$C581
TSTSTOP =	$C71C
iCCA4 =		$CCA4
iCCB8 =		$CCB8
iCE13 =		$CE13
iCE1C	=		$CE1C

iD57B =		$D57B
iD676 =		$D676
iD6D0 =		$D6D0

iE27D =		$E27D
iE2FA =		$E2FA
KPRCHAR =	$E3EA
KPRYMSG =	$E7DE
PIA1_PA =	$E810
PIA1_CRA =	$E811
PIA1_PB =	$E812
PIA2_PA =	$E820
PIA2_CRA =	$E821
PIA2_PB =	$E822
PIA2_CRB =	$E823
VIA_DRB =	$E840
VIA_IFR =	$E84D
VIA_T1CH =	$E845


iF83B =		$F83B
iF806 =		$F806
iF80C =		$F80C
iF82D =		$F82D
iF871 =		$F871
iF87F =		$F87F
iF8B9 =		$F8B9
iF88A =		$F88A
iF8BC =		$F8BC
iF8C4 =		$F8C4
iF913 =		$F913
iFBDC =		$FBDC
SETST =		$FBE5
iFD90 =		$FD90
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MOD WAS:
;F000       .byte $54, $4F, $4F, $20, $4D, $41, $4E, $59  ;too many
;F008             .byte $20, $46, $49, $4C, $45, $D3, $46, $49  ; fileSfi
;F010             .byte $4C, $45, $20, $4F, $50, $45, $CE, $46  ;le opeNf
;F018             .byte $49, $4C, $45, $20, $4E, $4F, $54, $20  ;ile not 
;F020             .byte $4F, $50, $45, $CE, $46, $49, $4C, $45  ;opeNfile
;F028             .byte $20, $4E, $4F, $54, $20, $46, $4F, $55  ; not fou
;F030             .byte $4E, $C4, $0D, $53, $45, $41, $52, $43  ;nDmsearc
;F038             .byte $48, $49, $4E, $47, $A0, $46, $4F, $52  ;hing for
;F040             .byte $A0, $0D, $50, $52, $45, $53, $53, $20  ; mpress 
;F048             .byte $50, $4C, $41, $59, $A0, $26, $20, $52  ;play & r
;F050             .byte $45, $43, $4F, $52, $44, $A0, $4F, $4E  ;ecord on
;F058             .byte $20, $54, $41, $50, $45, $20, $A3, $0D  ; tape #m
;F060             .byte $4C, $4F, $41, $C4, $0D, $57, $52, $49  ;loaDmwri
;F068             .byte $54, $49, $4E, $47, $A0, $0D, $56, $45  ;ting mve
;F070             .byte $52, $49, $46, $D9, $44, $45, $56, $49  ;rifYdevi
;F078             .byte $43, $45, $20, $4E, $4F, $54, $20, $50  ;ce not p
;F080             .byte $52, $45, $53, $45, $4E, $D4            ;resenT

iF000:
; I don't see any access "too many files" anywhere in entire rom-1 rom
; I checked all calls for kprymsg, kprdmode, yerrout - all seem to always
; have Y set and non-zero, even on branches
	.byt $23 ; # (too many files)
;--->11 bytes here
;-------------------------------------------------------------------------------
;-text could be moved-----------------------------------------------------------
PATCH_KSAVEPREPARE:
	JSR   LISTEN ;$F0BA
	LDA   SA
	JSR   SECOUT
	LDA   LADDR ;send the load address lo/hi
	JSR   IECOUT ;$F167
	LDA   LADDR+1
	JMP   IECOUT ;$F167 ;13 bytes
FILE_OPEN_ERROR: ;moved from $f00e to $f013 
; duplicate error message quickly by open1,8,8,"test":open1,8,8,"test" (make sure to then close file) close1
; confirmed only one use in entire codebase

	.byt $4F,$50,$45,$CE ; opeN (file open)
;--->5 bytes here
FILE_NOT_OPEN_ERROR: ; confirmed only one use in entire codebase
	;duplicate error message quickly by print#5,"hello" with 5 not open
iF017:
                    ;.byte $46,$49,$4c,$45,$20,$4e,$4f,$54   ;file not
                    ;.byt  $20,$4f,$50,$45,$ce               ; opeN
                    .byt  $46,$4e,$4f,$50,$ce; jdc changed to: fnopN (file not opeN)
;--->8 bytes here

PATCH_SA60:
	LDA   #$60
	.byt $2c ; bit operation to skip next LDA
PATCH_SA61:
	LDA   #$61
	STA   SA
	RTS ;8 bytes

FILE_NOT_FOUND_ERROR:
; confirmed only one use in entire codebase (odd one in the $cxxx space)
; duplicate error message quickly by sys 62841 ($f579)
iF024:
                  .byte $46, $49, $4C, $45, $20, $4E, $4F, $54  ;file not
                  .byte $20, $46, $4F, $55, $4E, $C4  ; founD

SSEARCHING: ; confirmed only one use in entire codebase
iF032:
                    .byt  $0d,$53,$45,$41,$52,$43,$48  ;^Msearch
SING: ; confirmed only one use in entire codebase
if039:
                    .byt  $49,$4e,$47,$a0                      ;inG
SFOR: ; confirmed only one use in entire codebase
                    .byt  $46,$4f,$52,$a0                  ;for 
                    .byt  $0d,$50,$52,$45,$53,$53,$20,$50  ;^Mpress p
                    .byt  $4c,$41,$59,$a0                  ;lay 
;-end---------------------------------------------------------------------------
;-------------------------------------------------------------------------------

iF04D:;(used once f87b)
                    .byt  $26,$20,$52,$45,$43,$4f,$52,$44  ;& record
                    .byt  $a0                              ; 
iF056:;(used once f845)
                    .byt  $4f,$4e,$20,$54,$41,$50,$45,$20  ;on tape
                    .byt  $a3                              ;#

;-------------------------------------------------------------------------------
;-text could be moved-----------------------------------------------------------
CRLOAD: ; confirmed only one use in entire codebase
iF05f:
                    .byt  $0d                              ;^M
LOAD_ERROR:
                    .byt  $4c,$4f,$41,$c4                  ;loaD
SWRITING: ; confirmed only one use in entire codebase
iF064:
                    .byt  $0d,$57,$52,$49,$54,$49,$4e,$47  ;^Mwriting
                    .byt  $a0                              ; 

CRVERIFY: ; confirmed only one use in entire codebase
iF06D:
                    .byt  $0d                              ;^M
VERIFY_ERROR: ; confirmed only one use in entire codebase
                    .byt  $56,$45,$52,$49,$46,$d9          ;verifY
DEVICE_NOT_PRESENT_ERROR: ; confirmed only TWO uses in entire codebase, both in this section
; The required I/0 device was not available for an 'OPEN', 'CLOSE', 'CMD', 'PRINT#', 'INPUT#', or'GET#'
; duplicate error message quickly by sys 63157 ($f6b5)
iF074:
                    .byt  $44,$45,$56,$4e,$d0              ;devnP (device not presenT)
;--->13 bytes here
PATCH_LDEND:
	JSR   UNTALK
	JSR   CLSIEC
	LDA   VERFL ;$020b
	RTS ;10 bytes
   BRK
	BRK
	BRK
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
NOT_INPUT_FILE_ERROR:
;An attempt was made to 'INPUT' or'GET' data from a file which was specified to be for output only.
; duplicate error message quickly by sys 63409 ($f7b1)
; confirmed only one use in entire codebase
                    .byte $4E, $4F, $54, $20, $49, $4E, $50, $55  ;not inp
                    .byte $54, $20, $46, $49, $4C, $C5  ;ut filE

;-end---------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;NOT_OUTPUT_FILE_ERROR:
iF094: ; used once f802
; duplicate error message quickly by sys63490 ($f802)
                    .byte $4E, $4F, $54, $20, $4F, $55, $54, $50  ;not outp
                    .byte $55, $54, $20, $46, $49, $4C, $C5  ;ut filE
;An attempt was made to 'PRINT' data to a file which was specified as input only

SFOUND: ; confirmed only one use in entire codebase
iF0A3:
                    .byt  $0d,$46,$4f,$55,$4e,$44,$a0      ;^Mfound 
SOK: ; USED TWICE, once not in this section $f857
iF0AA:
                    .byt  $0d,$4f,$4b,$8d                   ;^Mok^M
SREADY:
iF0AE: ; confirmed only one use in entire codebase
                    .byt  $0d,$52,$45,$41,$44,$59,$2e,$8d  ;^Mready.^M






;start of source
TALK:
	LDA   #$40
	BNE   ATNOUT ;ATNOUT: ;$F0BC

LISTEN:
	LDA   #$20
ATNOUT:
	PHA
	LDA   VIA_DRB				;$E840 NRFD hi
	ORA   #$02
	STA   VIA_DRB				;$E840 store
	LDA   #$3C
	STA   PIA2_CRA				;$E821 NDAC hi
	BIT   OUTFL; $021D
	BEQ   iF0E1					; if byte not saved, skip
	LDA   #$34
	STA   PIA1_CRA				;$E811 set EOI
	JSR   RIECOUT; $F0F1		; send last byte with EOI
	LDA   #$00
	STA   OUTFL; $021D			; clear buffer flag
	LDA   #$3C
	STA   PIA1_CRA; $E811		; clear EOI
iF0E1:
	PLA
	ORA   FA
	STA   OUTBUF ;$0222
iF0E7:
	LDA   VIA_DRB ;$E840		; wait DAV hi
	BPL   iF0E7
	AND   #$FB
	STA   VIA_DRB ;$E840		; ATN lo
RIECOUT:
	LDA   #$3C					; DAV hi
	STA   PIA2_CRB ;$E823
	LDA   VIA_DRB ;$E840
	AND   #$41					; NRFD and NDAC
	CMP   #$41
	BEQ   iF142					; device not present
	LDA   OUTBUF ;$0222
	EOR   #$FF
	STA   PIA2_PB; $E822		; send data to bus
iF107:
	BIT   VIA_DRB ;$E840
	BVC   iF107					; wait NRFD hi
	LDA   #$34
	STA   PIA2_CRB ;$E823		; DAV lo
	LDA   #$FF
	STA   VIA_T1CH ;$E845		; set timer for 65ms
iF116:
	LDA   VIA_DRB ;$E840
	BIT   VIA_IFR ;$E84D
	BVS   iF13B					; timer end -> error
	LSR
	BCC   iF116					; wait NDAC hi
iF121:
	LDA   #$3C					
	STA   PIA2_CRB				;$E823 DAV hi
	LDA   #$FF
	STA   PIA2_PB				;$E822 clear bus data lines
	RTS

SECOUT:
	STA   OUTBUF					;$0222 send secondary address
	JSR   RIECOUT; $F0F1
iF132:
	LDA   VIA_DRB				;$E840
	ORA   #$04
	STA   VIA_DRB				;$E840 ATN hi
	RTS
iF13B:
	LDA   #$01					; time-out error (out)
iF13D:
	JSR   SETST; $FBE5
	BNE   iF121					; always
iF142:
	LDA   #$80					; device not present error
	BMI   iF13D					; always
iF146:
	LDA   #$02					; time-out error (in)
	JSR   SETST					;$FBE5 set status
iF14B:
	LDA   VIA_DRB ;$E840
	AND   #$FD
	STA   VIA_DRB				;$E840 NRFD lo
	LDA   #$34
	STA   PIA2_CRA				;$E821 NDAC lo
	LDA   #$0D
	RTS
SECTALK:
	STA   OUTBUF ;$0222		; send secondary talk
	JSR   RIECOUT; $F0F1
iF161:
	JSR   iF14B					; NRFD, NDAC lo
	JMP   iF132					; ATN hi
IECOUT:
	BIT   OUTFL; $021D
	BMI   iF171					; no byte buffered -> skip
	DEC   OUTFL					;$021D set buffer flag
	BNE   iF176
iF171:
	PHA
	JSR   RIECOUT; $F0F1
	PLA
iF176:
	STA   OUTBUF					;$0222 save byte in buffer
	RTS
UNTALK:
	LDA   #$5F
	BNE   iF180
UNLISTEN:
	LDA   #$3F
iF180:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	NOP ;mod was STA FA
	NOP ;mod
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JSR   ATNOUT ;$F0BC
	BNE   iF132					; always (?) -> ATN hi 
IECIN:
	LDA   #$34
	STA   PIA2_CRA				;$E821 NDAC lo
	LDA   VIA_DRB ;$E840
	ORA   #$02
	STA   VIA_DRB				;$E840 NRFD hi
	LDA   #$FF
	STA   VIA_T1CH				;$E845 timer for 65ms
iF199:
	BIT   VIA_IFR ;$E84D
	BVS   iF146					; timer end -> error
	BIT   VIA_DRB ;$E840
	BMI   iF199					; wait DAV lo
	LDA   VIA_DRB ;$E840
	AND   #$FD
	STA   VIA_DRB ;$E840		; NRFD lo
	BIT   PIA1_PA ;$E810
	BVS   iF1B5					; EOI set?
	LDA   #$40					; yes
	JSR   SETST					;$FBE5 set status
iF1B5:
	LDA   PIA2_PA				;$E820 load bus data
	EOR   #$FF
	PHA
	LDA   #$3C
	STA   PIA2_CRA				;$E821 NDAC hi
iF1C0:
	BIT   VIA_DRB ;$E840
	BPL   iF1C0					; wait DAV hi
	LDA   #$34
	STA   PIA2_CRA ;$E821		; NDAC lo
	PLA
	RTS

KGET:
	LDA   #$00					; GET call
	STA   STATUS					;$020C clear status
	LDA   CURIDEV ;$0263
	BNE   iF1F1
	LDA   NKEYS					;$020D get from keyboard
	BEQ   iF22C					; number of keys in buffer zero ->RTS
	SEI
	JMP   iE27D
KBASIN:
	LDA   CURIDEV				;$0263 BASIN call
	BNE   iF1F1					; not kebaord then as GET
	LDA   xE2
	STA   x0221
	LDA   LINENO ;$F5
	STA   x0220
	JMP   iE2FA
iF1F1:
	CMP   #$03					; get from screen?
	BNE   iF200
	STA   x0260
	LDA   SECADDRHI; $F2
	STA   x021E
	JMP   iE2FA




iF200:
	BCS   iF227					; dev > 3 (IEEE) -> f227
	STX   x0261					; get from tape
iF205:
	JSR   iF82D
	BNE   iF218
	JSR   iF87F
	LDY   #$00
	TYA
	LDX   FA
	STA   x0270,X
	JMP   iF205
iF218:
	LDA   (TAPEBUFFER),Y ;$F3
	BNE   iF223
	LDA   #$40
	JSR   SETST; $FBE5
	BNE   iF205
iF223:
	LDX   x0261
	RTS
iF227:
	LDA   STATUS					;$020C get from IEEE488
	BEQ   iF22D
iF22C:
	RTS
iF22D:
	JMP   IECIN ;$F187
KBSOUT:
	PHA
	LDA   CURODEV; $0264
	BNE   iF239
	JMP   iC379					; keyboard -> error
iF239:
	CMP   #$03
	BNE   iF241					; not screen ->
	PLA
	JMP   KPRCHAR
iF241:
	BMI   iF247					; tape ->
	PLA
	JMP   IECOUT					;$F167 send to IEEE488
iF247:
	PLA
iF248:
	STA   xE9
	CMP   #$1D
	BNE   iF253
	INC   x026A
	BEQ   iF22C
iF253:
	CMP   #$0A
	BEQ   iF22C
	PHA
	TXA
	PHA
	TYA
	PHA
	JSR   iF82D
	BNE   iF273
	JSR   iF8B9
	LDX   FA
	LDA   #$01
	STA   x0270,X
	JSR   iF5E3
	LDA   #$02
	STA   (TAPEBUFFER),Y ;$F3
	INY
iF273:
	LDA   xE9
	STA   (TAPEBUFFER),Y ;$F3
iF277:
	PLA
	TAY
	PLA
	TAX
	PLA
	RTS

KCLRCH:
	LDA   CURODEV				;$0264 CLRCH, clear I/O 
	BEQ   iF28B
	CMP   #$03
	BEQ   iF28B
	BMI   iF28B
	JSR   UNLISTEN ;$F17E
iF28B:
	LDA   CURIDEV ;$0263
	BEQ   iF299
	CMP   #$03
	BEQ   iF299
	BMI   iF299
	JSR   UNTALK ;$F17A
iF299:
	LDA   #$00
	STA   CURIDEV ;$0263
	LDA   #$03
	STA   CURODEV; $0264
	RTS

KCLALL:
	LDA   #$00
	STA   NFILES ;$0262
	BEQ   KCLRCH

SLOGFN:
	LDX   NFILES					;$0262 set file parameter of log. fn in AC
iF2AE:
	DEX
	BMI   iF2C7
	CMP   LFTAB,X
	BEQ   iF2C7
	BNE   iF2AE

SETFPAR:
	LDA   LFTAB,X
	STA   LFN
	LDA   FATAB,X
	STA   FA
	LDA   SATAB,X
	STA   SA
iF2C7:
	RTS

KCLOSE:
	JSR   OPENPAR				;$F4D4 Close
	LDA   LFN
	JSR   SLOGFN ;$F2AB
	BNE   iF329
	JSR   SETFPAR ;$F2B8
	TXA
	PHA
	LDA   FA						; device number
	BEQ   iF30A					; keyboard ->
	CMP   #$03
	BEQ   iF30A					; screen ->
	BCS   iF307					; IEEE488 ->
	LDA   SA						; Tape
	BEQ   iF30A
	JSR   iF667
	LDX   #$02
iF2EA:
	LDA   xF304,X
	JSR   iF248
	DEX
	BPL   iF2EA
	JSR   iF8B9
	LDA   SA
	CMP   #$02
	BNE   iF30A
	LDA   #$05
	JSR   iF5ED
	JMP   iF30A

xF304:
                    .byt $0D, $31, $00			;m1.

iF307:
	JSR   CLSIEC					; close IEEE488
iF30A:
	PLA
	TAX
	DEC   NFILES ;$0262
	CPX   NFILES ;$0262
	BEQ   iF329
	LDY   NFILES ;$0262
	LDA   LFTAB,Y
	STA   LFTAB,X
	LDA   FATAB,Y
	STA   FATAB,X
	LDA   SATAB,Y
	STA   SATAB,X
iF329:
	RTS

CHKSTP:
	LDA   KEYMSK ;$0209
	CMP   #LFN
	BNE   iF338
	PHP
	LDA   #$00
	STA   NKEYS; $020D
	PLP
iF338:
	RTS

IECABORT:
	JSR   CHKSTP; $F32A
	JMP   TSTSTOP ;$C71C

KPRDMODE:
	LDA   xCA						; print kmsg in direct mode only
	BNE   iF338
	JMP   KPRYMSG; $E7DE

KLOAD:
	LDA   #$00
	STA   VERFL

IECLOAD:
	JSR   GETFPAR ;$F433
	LDA   #$FF					; wait till no key pressed
iF350:
	CMP   KEYMSK ;$0209
	BNE   iF350
	CMP   KEYMSK ;$0209
	BNE   iF350
	LDA   #$04					; default load address to $0400
	STA   LADDR+1
	LDA   #$00
	STA   LADDR
	LDA   FA						; check device address
	BNE   iF369
iF366:
	JMP   iCE1C					; Syntax error (keyboard, dev. 0)
iF369:
	CMP   #$03
	BEQ   iF366					; Syntax error (screen, dev. 3)
	BCC   iF3A5					; load from tape (device 1,2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MODS IN HERE:
; ORIGINAL:
; F36F  20 1C F7             JSR iF71C      ; look for unused sec. address
; F372  20 FF F3             JSR PRSRCH     ; print "searching for ..."
; F375  20 62 F4             JSR IECOPEN   
; F378  20 B6 F0             JSR TALK      
; F37B  20 22 F4             JSR iF422      ; print "loading/verifying" 
; F37E  20 39 F3  iF37E      JSR IECABORT   ; check stop key
; F381  20 87 F1             JSR IECIN     
; F384  AE 0C 02             LDX STATUS    
; F387  30 43                BMI LDEND      ; status EOI then end

	JSR   PATCH_SA60		;MOD! (set SA to #$60 - read secondary address)
	JSR   PRSRCH ;$F3FF		; print "searching for ..."
	JSR   IECOPEN ;$F462
	JSR   PATCH_KLOADPREPARE ; patch was JSR TALK (talk, sectalk, populate the load address lo/hi)
	JSR   iF422					; print "loading/verifying"
iF37E:
	JSR   IECABORT				;$F339 check stop key
	BIT   STATUS ;$020C
	BVS	LDEND
	JSR   IECIN ;Andre's code had this line and next swapped, causing growth of one byte on load
	;BVS   LDEND					; status EOI then end
; MODS END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDY   VERFL ;$020b
	BEQ   iF39A
	DEY
	CMP   (LADDR),Y				; compare when verifying
	BEQ   iF39C
	LDX   #$02
	STX   STATUS ;$020C
	BNE   iF39C
iF39A:
	STA   (LADDR),Y				; store when loading
iF39C:
	INC   LADDR
	BNE   iF37E
	INC   LADDR+1
	JMP   iF37E
iF3A5:
	JSR   iF667					; load from tape
	JSR   iF83B
	JSR   PRSRCH ;$F3FF
iF3AE:
	LDA   FNLEN
	BEQ   iF3BA
	JSR   iF495
	BNE   iF3BF
iF3B7:
	JMP   iF579
iF3BA:
	JSR   iF5AE
	BEQ   iF3B7
iF3BF:
	CPX   #$01
	BNE   iF3AE
	JSR   iF64D
	JSR   iF422
	JSR   iF88A
LDEND:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MOD!
	JSR   PATCH_LDEND ;MOD! was LDA VERFL (untalk, clsiec, lda verfl)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	BNE   iF421			; verify -> rts
	JSR   iF913			; check break status
	LDA   STATUS ;$020C
	AND   #$10
	BEQ   iF3E5			; ok -> 
	LDY   #$00
	STY   NKEYS; $020D
	LDY   #<LOAD_ERROR			; #$60 "Load error"
	JMP   YERROUT ;$F57B
iF3E5:
	LDY   #<SREADY			; #$ae print "READY."
	JSR   KPRDMODE ;$F33F
	LDA   xCA
	BNE   iF3F9

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDA   LADDR+1 ;was EADDR+1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	STA   VARSTART+1 ;$7D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDA   LADDR ;was EADDR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	STA   VARSTART ;$7C
	JMP   iC430
iF3F9:
	JSR   iC59A			; (c9) = (7a)-1
	JMP   iC581			; clear stack, restore BASIC, return

PRSRCH:
	LDA   xCA				; print "searching for ..."
	BNE   iF421			; test direct mode?
	LDY   #<SSEARCHING ;#$32
	JSR   KPRYMSG		;$E7DE print "searching"
	LDA   FNLEN
	BEQ   iF421			; filename length 0 ->RTS
	LDY   #<SFOR ;#$3d
	JSR   KPRYMSG		;$E7DE print "for"
iF411:
	LDY   FNLEN			; print filename
	BEQ   iF421
	LDY   #$00
iF417:
	LDA   (OSPTR),Y
	JSR   KPRCHAR
	INY
	CPY   FNLEN
	BNE   iF417
iF421:
	RTS
iF422:
	LDY   #<CRLOAD			; #$5f print loading/verifying 
	LDA   VERFL ;$020b
	BEQ   iF42B
	LDY   #<CRVERIFY ;#$6d
iF42B:
	JSR   KPRDMODE ;$F33F
	LDY   #<SING ; #$39 ing
	JMP   KPRDMODE ;$F33F

GETFPAR:
	LDX   #$00			; Get LOAD parameter
	STX   STATUS ;$020C
	STX   EADDR
	STX   FNLEN
	STX   SA
	INX
	STX   FA				; default to tape (FA=1)
	LDA   #$04
	STX   EADDR+1
	JSR   ENDCMD ;$F515
	JSR   iF504			; get filename
	JSR   ENDCMD ;$F515
	JSR   CHKCOMMA ;$F45C
	STX   FA
	JSR   ENDCMD ;$F515
	JSR   CHKCOMMA ;$F45C
	STX   SA				; secondary address
iF45B:
	RTS

CHKCOMMA:
	JSR   iF51D
	JMP   iD676

IECOPEN:
	LDA   SA				; open IEC file
	BMI   iF45B
	LDY   FNLEN
	BEQ   iF45B
	JSR   LISTEN ;$F0BA
	LDA   SA
	ORA   #$40
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	NOP		;MOD WAS STA SA
	NOP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	ORA   #$F0
	JSR   SECOUT ;$F12C
	LDA   STATUS ;$020C
	BPL   iF482
iF47D:
	LDY   #<DEVICE_NOT_PRESENT_ERROR			; #$74 device not present error
	JMP   YERROUT ;$F57B
iF482:
	LDA   FNLEN
	BEQ   iF492
	LDY   #$00
iF488:
	LDA   (OSPTR),Y
	JSR   IECOUT ;$F167
	INY
	CPY   FNLEN
	BNE   iF488
iF492:
	JMP   UNLISTEN ;$F17E

iF495:
	JSR   iF5AE			; Tape stuff
	BEQ   iF4BA
	LDY   #$05
	STY   x0268
	LDY   #$00
	STY   xE9
iF4A3:
	CPY   FNLEN
	BEQ   iF4B9
	LDA   (OSPTR),Y
	LDY   x0268
	CMP   (TAPEBUFFER),Y ;$F3
	BNE   iF495
	INC   xE9
	INC   x0268
	LDY   xE9
	BNE   iF4A3
iF4B9:
	TYA
iF4BA:
	RTS

KVERIFY:
	LDA   #$01
	STA   VERFL ;$020b
	JSR   IECLOAD ;$F34B
	LDA   STATUS ;$020C
	AND   #$10
	BEQ   iF4CF
	LDY   #<VERIFY_ERROR						;#$6e verify error
	JMP   YERROUT ;$F57B
iF4CF:
	LDY   #<SOK ;#$AA
	JMP   KPRYMSG; $E7DE

OPENPAR:
	LDX   #$00
	STX   SA
	STX   STATUS ;$020C
	STX   FNLEN
	INX
	STX   FA
	JSR   iF522
	JSR   iD676
	STX   LFN
	JSR   ENDCMD ;$F515
	JSR   CHKCOMMA ;$F45C
	STX   FA
	CPX   #$03
	BCC   iF4F6
	DEC   SA
iF4F6:
	JSR   ENDCMD ;$F515
	JSR   CHKCOMMA ;$F45C
	STX   SA
	JSR   ENDCMD ;$F515
	JSR   iF51D
iF504:
	JSR   iCCB8
	JSR   iD57B
	STA   FNLEN
	LDA   IINDEX
	STA   OSPTR
	LDA   IINDEX+1
	STA   OSPTR+1
	RTS

ENDCMD:
	JSR   iC8
	BNE   iF51C
	PLA
	PLA
iF51C:
	RTS

iF51D:
	LDA   #$2C
	JSR   iCE13
iF522:
	JSR   iC8
	BNE   iF51C
	JMP   iCE1C
	JSR   OPENPAR			;$F4D4 Kernal Open
	LDA   LFN
	BNE   iF534
	JMP   iCE1C
iF534:
	JSR   SLOGFN ;$F2AB
	BNE   iF53D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDY   #<FILE_OPEN_ERROR ;was #$0e
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
iF53B:
	BNE   YERROUT ;$F57B
iF53D:
	LDX   NFILES				;$0262 file not in use already
	LDY   #$00
	STY   STATUS ;$020C
	CPX   #$0A
	BEQ   iF53B				; file table full -> lock!
	INC   NFILES ;$0262
	LDA   LFN					; save file parameter
	STA   LFTAB,X
	LDA   SA
	STA   SATAB,X
	LDA   FA
	STA   FATAB,X
	BEQ   iF5AD				; check devices
	CMP   #$03
	BEQ   iF5AD
	BCC   iF566
	JMP   IECOPEN			;$F462 iecopen
iF566:
	LDA   SA					; Tape
	BNE   iF592
	JSR   iF83B
	JSR   PRSRCH ;$F3FF
	LDA   FNLEN
	BEQ   iF58B
	JSR   iF495
	BNE   iF59A
iF579:
	LDY   #<FILE_NOT_FOUND_ERROR ;#$24
YERROUT:
	LDA   #$0D
	JSR   KPRCHAR
	LDA   #$3F
	JSR   KPRCHAR
	JSR   KPRYMSG; $E7DE
	JMP   iC379
iF58B:
	JSR   iF5AE
	BEQ   iF579
	BNE   iF59A
iF592:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JSR   iF871    
	LDA   #$04
	JSR   iF5ED
iF59A:
	LDX   FA
	LDA   #$BF
	LDY   SA
	BEQ   iF5AA
	JSR   iF5E3
	LDA   #$02
	STA   (TAPEBUFFER),Y ;$F3
	TYA
iF5AA:
	STA   x0270,X
iF5AD:
	RTS
iF5AE:
	LDA   VERFL ;$020b
	PHA
iF5B2:
	JSR   iF87F
	LDY   #$00
	LDA   (TAPEBUFFER),Y ;$F3
	CMP   #$05
	BEQ   iF5DD
	CMP   #$01
	BEQ   iF5C5
	CMP   #$04
	BNE   iF5B2
iF5C5:
	TAX
	LDA   xCA
	BNE   iF5DB
	LDY   #<SFOUND			 	; #$a3 "found"
	JSR   KPRYMSG; $E7DE
	LDY   #$05
iF5D1:
	LDA   (TAPEBUFFER),Y ;$F3
	JSR   KPRCHAR
	INY
	CPY   #$15
	BNE   iF5D1
iF5DB:
	LDY   #$01
iF5DD:
	PLA
	STA   VERFL ;$020b
	TYA
	RTS

iF5E3:
	LDY   #$BF
	LDA   #$20
iF5E7:
	STA   (TAPEBUFFER),Y ;$F3
	DEY
	BNE   iF5E7
	RTS

iF5ED:
	STA   xE9
	LDA   LADDR+1
	PHA
	LDA   LADDR
	PHA
	LDA   EADDR+1
	PHA
	LDA   EADDR
	PHA
	JSR   iF5E3
	LDA   xE9
	STA   (TAPEBUFFER),Y ;$F3
	INY
	LDA   LADDR
	STA   (TAPEBUFFER),Y ;$F3
	INY
	LDA   LADDR+1
	STA   (TAPEBUFFER),Y ;$F3
	INY
	LDA   EADDR
	STA   (TAPEBUFFER),Y ;$F3
	INY
	LDA   EADDR+1
	STA   (TAPEBUFFER),Y ;$F3
	INY
	STY   x0268
	LDY   #$00
	STY   xE9
iF61E:
	LDY   xE9
	CPY   FNLEN
	BEQ   iF632
	LDA   (OSPTR),Y
	LDY   x0268
	STA   (TAPEBUFFER),Y ;$F3
	INC   xE9
	INC   x0268
	BNE   iF61E
iF632:
	JSR   iF67D
	JSR   iF913
	LDA   #$69
	STA   SECDATA ;$0279
	JSR   iF8C4
	PLA
	STA   EADDR
	PLA
	STA   EADDR+1
	PLA
	STA   LADDR
	PLA
	STA   LADDR+1
	RTS
iF64D:
	JSR   iF913
	LDX   #$00
	LDY   #$01
iF654:
	LDA   (TAPEBUFFER),Y ;$F3
	STA   SADDR,X
	INX
	INY
	CPX   #$04
	BNE   iF654
	LDA   SADDR
	STA   LADDR
	LDA   SADDR+1
	STA   LADDR+1
	RTS
iF667:
	LDA   #$7A
	STA   TAPEBUFFER
	LDA   #$02
	STA   TAPEBUFFER+1
	LDA   FA
	LSR
	BCS   iF67C
	LDA   #$3A
	STA   TAPEBUFFER
	LDA   #$03
	STA   TAPEBUFFER+1
iF67C:
	RTS

iF67D:
	JSR   iF913
	JSR   iF667
	LDA   TAPEBUFFER
	STA   LADDR
	CLC
	ADC   #$C0
	STA   EADDR
	LDA   TAPEBUFFER+1
	STA   LADDR+1
	ADC   #$00
	STA   EADDR+1
	RTS

KSYS:
	JSR   iCCA4
	JSR   iD6D0
	JMP   (x08)

KSAVE:
	JSR   GETFPAR ;$F433
	LDA   VARSTART ;$7C
	STA   EADDR
	LDA   VARSTART+1 ;$7D
	STA   EADDR+1
	LDA   #$04					; default save from $0400
	STA   LADDR+1
	LDA   #$00
	STA   LADDR
	LDA   FA
	BNE   iF6BA
iF6B5:
	LDY   #<DEVICE_NOT_PRESENT_ERROR	; #$74 keyboard -> device not present
	JMP   YERROUT ;$F57B
iF6BA:
	CMP   #$03
	BEQ   iF6B5					; screen -> device not present
	BCC   iF6F6					; tape ->
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JSR   PATCH_SA61 ; MOD - was JSR $F71C  (set sa to #$61 - write secondary address)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JSR   IECOPEN ;$F462
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JSR   PATCH_KSAVEPREPARE ; MOD - was JSR LISTEN $F0BA (listen, secout, send the load address lo/hi)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	LDY   #$00
	JSR   iFBDC					; LADR -> SADR
iF6CE:
	JSR   iFD90					; compare with end address
	BEQ   iF6E3
	LDA   (SADDR),Y
	JSR   IECOUT ;$F167
	JSR   IECABORT ;$F339
	INC   SADDR
	BNE   iF6CE
	INC   SADDR+1
	BNE   iF6CE
iF6E3:
	JSR   UNLISTEN ;$F17E

CLSIEC:
	BIT   SA
	BMI   iF735
	JSR   LISTEN ;$F0BA
	LDA   SA
	AND   #LFN
	ORA   #$E0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	JMP   PATCH_KSAVEDONE ;MOD - was JMP SECOUT (secout, then unlisten)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
iF6F6:
	JSR   iF667					; Tape
	JSR   iF871    
	LDA   xCA
	BNE   iF708
	LDY   #<SWRITING ;#$64
	JSR   KPRYMSG; $E7DE
	JSR   iF411
iF708:
	LDA   #$01
	JSR   iF5ED
	JSR   iF8BC
	LDX   SA ;SA
	BEQ   iF735
	DEX
	BEQ   iF735
	LDA   #$05
	JMP   iF5ED
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 25 bytes, was:
;iF71C: ;due to patch, not called anymore anywhere in rom
;	LDX   #$00					; look for free sec. address
;	STX   SA
;iF720: ;due to patch, not called anymore anywhere in rom
;	LDX   #$00
;	INC   SA
;iF724: ;due to patch, not called anymore anywhere in rom
;	CPX   NFILES ;$0262
;	BEQ   iF787					; -> RTS
;	LDA   SATAB,X
;	AND   #$1F
;	CMP   SA
;	BEQ   iF720
;	INX
;	BNE   iF724
PATCH_KSAVEDONE: ;call secout, then unlisten
	JSR   SECOUT ;$F12C
	JMP   UNLISTEN ;$F17E-6 bytes
	BRK
PATCH_KLOADPREPARE:
	JSR   TALK
	LDA   SA
	JSR   SECTALK
	JSR   IECIN ;populate the load address lo/hi
	STA   LADDR
	JSR   IECIN
	STA   LADDR+1 ;18 bytes

iF735:
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Update Jiffy Clock
iF736:
	LDA   TIMER1
	ADC   #$01
	STA   TIMER1				; Jiffy clock correction: 623rd 1/60 sec does not increment time
	BCC   iF743
	INC   TIMER1+1
iF743:
	CMP   #$6F
	BNE   iF74E
	LDA   TIMER1+1
	CMP   #$02
	BEQ   iF774
iF74E:
	INC   TCLOCK+2	
	BNE   iF75B
	INC   TCLOCK+1
	BNE   iF75B
	INC   TCLOCK				; Real-Time Jiffy Clock (approx) 1/60 Sec
iF75B:
	LDX   #$00
iF75D:
	LDA   TCLOCK,X
	CMP   xF788,X
	BCC   iF77C
	INX
	CPX   #$03
	BNE   iF75D
	LDA   #$00
iF76C:
	STA   STACK+255,X
	DEX
	BNE   iF76C
	BEQ   iF77C
iF774:
	LDA   #$00
	STA   TIMER1				; Jiffy clock correction: 623rd 1/60 sec does not increment time 
	STA   TIMER1+1
iF77C:
	LDA   PIA1_PB ;$E812
	CMP   PIA1_PB ;$E812
	BNE   iF77C
	STA   KEYMSK				;$0209  Flag: STOP key / RVS key
iF787:
	RTS
xF788:
	.byt $4F, $1A, $01				;oza

; chkin	Set Input Device

KCHKIN:
	PHA
	TXA
	PHA
	TYA
	PHA
	LDA   #$00
	STA   STATUS						;$020C clear status
	TXA
	JSR   SLOGFN ;$F2AB
	BEQ   iF7A0
iF79B:
	LDY   #<FILE_NOT_OPEN_ERROR						; #$17 "file not open"
iF79D:
	JMP   YERROUT ;$F57B
iF7A0:
	JSR   SETFPAR ;$F2B8
	LDA   FA													; Current Device Number
	BEQ   iF7B5
	CMP   #$03
	BEQ   iF7B5
	BCS   iF7BB					; IEEE488 ->
	LDX   SA						; Tape - Current Secondary Address
	BEQ   iF7B5
	LDY   #<NOT_INPUT_FILE_ERROR ;#$86
	BNE   iF79D
iF7B5:
	STA   CURIDEV				;$0263 keyboard -> almost finished
	JMP   iF277					; restore registers, return
iF7BB:
	PHA
	JSR   TALK ;$F0B6
	LDA   SA
	BPL   iF7C9
	JSR   iF161					; NRFD, NDAC lo, ATN hi
	JMP   iF7D0
iF7C9:
	AND   #$1F
	ORA   #$60
	JSR   SECTALK
iF7D0:
	LDA   STATUS ;$020C
	BPL   iF7D8
	JMP   iF47D					; "device not present"
iF7D8:
	PLA
	JMP   iF7B5
	PHA
	TXA
	PHA
	TYA
	PHA
	LDA   #$00
	STA   STATUS ;$020C
	LDA   #$FF
	STA   x026A
	TXA
	JSR   SLOGFN ;$F2AB
	BNE   iF79B
	JSR   SETFPAR ;$F2B8
	LDA   FA						; check device
	BEQ   iF79B					; keyboard -> file not open
	CMP   #$03
	BEQ   iF806					; screen -> almost finished
	BPL   iF80C					; IEEE488 ->
	LDX   SA						; Tape
