.feature	c_comments

;-------------------------------------------------------------------------------
; ROM1 DISK MAGIC V1.2 by Jason Cook
;
; For updates: https://hub.inktada.com/channel/rom1diskmagic
;
; This code's concept was derived from the original IEEE-488 ROM 1 patch 
; written by André Fachat and ROM 1 disassembly.
; 
; This file is purposely self contained with no dependencies.
; 
; Like André's version, LOAD and VERIFY mostly returns OK. You must
; review the ST (status) basic variable to get the result or read location
; $20C. AND #$40 (bit 6) indicates end of file reached, which is a good
; indicator success. You may also want to check any of the bits 4, 5, 7 set, 
; as any set would indicate error.
; Quick test BASIC:
;   IF ((ST) AND 64)>0 THEN REM SUCCESS CODE HERE
; Quick test assembler:
;   LDA $20C
;   AND #$40 ;(bit 6) indicates end of file reached, NON ZERO means success.
;
; Save is less reliable, but ST=0 is the best indicator of success. Review
; status bits for PET.
;
; While BASIC programs should work fine, if you want to use the upgraded SAVE
; and LOAD in assembly language programs, first test to see if the patch has
; been applied by checking $c2+$2=#$03,
; $c2+$3=#$2c, and tapebuffer2+$a=#$03 
; 1.2 CHANGE - UNFORTUNATELY CHECK tapebuffer2+$8 HAD TO BE CHANGED TO tapebuffer2+$a
;
; TEST FOR PATCH EXAMPLE:
;   tapebuffer2 = $33a
;     LDA $C2+$2
;     CMP #$03
;     BNE NOTPATCHED
;     LDA $C2+$3
;     CMP #$2C
;     BNE NOTPATCHED
;     LDA tapebuffer2+$a
;     CMP #$03
;     BNE NOTPATCHED
;   PATCH IS IDENTIFIED AND ACTIVE
;     
; On test success, JMP addresses are as follows:
;
; JSR tapebuffer2+0: LOAD/VERIFY low level address.
;                Make sure to initialize.
;                FA(device number)
;                SA(secondary address)
;                LFN(logical file number)
;                FNLEN(file length)
;                OSPTR(filename)
;                VERFL(0=LOAD)
;
; JSR tapebuffer2+$5: (First TEST tapebuffer2+$8 = #$20, SAVE code enabled.)
;                SAVE low level address.
;                Make sure to initialize.
;                FA(device number)
;                SA(secondary address)
;                LFN(logical file number)
;                FNLEN(file length)
;                OSPTR(filename)
;                SADDR(start address of save)
;                EADDR(end address of save)
;
; For reference, native equivalents to above JMP locations:
; rom1load =				$f362
; rom2load =				$f322
; rom4load =				$f356
;
; rom1save =				$f6b1
; rom2save =				$f6a4
; rom4save =				$f6e3
;
; * rom2 and rom 4 have different locations for
;   FA, SA, LFN, FNLEN, OSPTR, SADDR, EADDR
;   rom2 and rom4 share the same locations for those variables.
;
; assembler used: ca65 https://cc65.github.io/doc/cl65.html :
; cl65 -o THISFILENAME.prg -t none -l THISFILENAME.lst THISFILENAME.s
;
;-------------------------------------------------------------------------------


;program must be in memory for verify to work!
;verify success st=66
;verify fail st=2

;;;
; constants
SPACE = $20 ; character " "
COLON = $3A ; character ":"
BACKSPACE = $9D
VARSTART =	$7C	;124
HIRAM = $86	;134
LOAD = $93
SAVE = $94
VERIFY = $95
DEVICE_NOT_PRESENT_ERROR	= $74 ;keyboard -> device not present
xCA	= $CA

screenxrom1	=	$e2
screenyrom1	=	$f5

KEYMSK =		$0209
VERFL =		$020B
STATUS =		$020C	;Kernal I/O Status Word: ST


CLR		= $c770
iCE1C		= $CE1C
TALK		= $F0B6
LISTEN	= $F0BA
SECOUT	= $F12C
SECTALK	= $F15B
IECOUT	= $F167
UNTALK	= $F17A
iF182		= $F182
IECIN		= $F187
IECABORT	= $F339
iF35A		= $F35A	;load entry point for devices less than 4
iF389		= $F389
iF3A5		= $F3A5
LDEND		= $F3CC
iF3D1		= $F3D1
PRSRCH	= $F3FF
iF422		= $F422
GETFPAR	= $F433
IECOPEN	= $F462
iF473		= $F473
YERROUT	= $F57B
DNPR		= $F47D ;device not present error
iF6A1		= $F6A1 ;save entry point for devices less than 4
iF6C9		=$F6C9 ;save the data byte by byte
;iF6CE		= $F6CE
iF6E3		= $F6E3
CLSIEC	= $F6E6
iF6F6		= $F6F6
iFBDC		= $FBDC
iFD90		= $FD90
cinn		= $ffcf
cout		= $ffd2
getchar	= $ffe4
;;;
; zero page
iC2 = $C2 ; begin of character interpreter
Z = $c9
MYZERO=iC2+5 ; my zero page locations

FNLEN =		$EE
SADDR =		$E3	;227
EADDR =		$E5	;229
xE9 =			$E9

LFN =			$EF	;Logical File Number
SA =			$F0	;Current Secondary Address
FA	=			$F1	;Current Device Number
LINENO =		$F5
LADDR =		$F7	;247
OSPTR =		$F9

tapebuffer1	= $27a	;634-825 (192 bytes)
tapebuffer2	= $33a;$33a-$3f9	;826-1017 (192 bytes)(both=384)


;routine to patch c2-d9
;iC2:
; inc Z; ;($c9) increment character pointer low byte
; bne @ovhi
; inc Z+1; ;($ca) increment character pointer high byte
;@ovhi:
;GET:
; lda $400 ;get the character from address
; cmp #COLON
; bcs END
; cmp #SPACE
; beq iC2
; sec
; sbc #$30
; sec
; sbc #$d0
;END:
; rts



mainentry=	$401



	.org $03fe
.segment "CODE"
startcode:
	.byt $00,04
	.byt $00
;.segment	"EXEHDR"
	;12 bytes
	.byt $0b,$04,$e7,$07,$9e,$31,$30,$33,$37,$00,$00,$00


initialize:
;.segment "CODE"

jmp init
MESSAGEBLOCK:
PROGRAMNAME:
	;		   "0123456789012345678901234567890123456789"
	.byt	18," ROM1 DISK MAGIC V1.2     BY JASON COOK ",146
	;		"0123456789012345678901234567890123456789"
	.byt	"   DERIVED FROM: ANDRE FACHAT & ROM CODE",13,0
NOTROM1:
	.byt	"NOT A ROM 1 SYSTEM, NO NEED FOR PATCH.",0
ROM1PATCHED:
	.byt	"ROM 1 ALREADY PATCHED. NO NEED TO APPLY",13
	.byt	"SOFTWARE PATCH.",0
ALREADYPATCHED:
	.byt	"PATCH ALREADY APPLIED. UNDO ENTIRE PATCH"
	.byt	"BY RESET SYS 64824.",13,13,0
SAVEQUESTION:
	.byt	"DO YOU NEED DISK SAVE FUNCTIONALITY?",0
SAVEDISABLED:
	.byt	"N",13,13
	.byt	"LOAD & VERIFY PATCHED. SAVE NOT PATCHED.",0
SAVEENABLED:
	.byt	"Y",13,13
	.byt	"LOAD,SAVE,& VERIFY PATCHED.",13,0
RERUN:
	.byt	"RELOAD & RUN TO ADJUST SAVE PATCH.",0
PATCHDETAIL:
	;		"0123456789012345678901234567890123456789"
	.byt	"FINALLY! THE ABILITY FOR THE ROM1 SYSTEM"
	.byt	"TO USE DISK (IEEE-488) DEVICES!",13
	.byt	13
	.byt	"LOAD AND VERIFY MOSTLY RETURN OK.",13
	.byt	"CHECK ((ST) AND 64) > 0 FOR SUCCESS.",13
	.byt	13
	.byt	"THE PATCH REQUIRES SPACE RESERVED FOR",13
	.byt	"TAPE #2,SO IT IS DISABLED. ADDITIONALLY,"
	.byt	"IF SAVE IS UTILIZED, IT USES 75 BYTES",13
	.byt	"AT THE TOP OF PROGRAM SPACE. UNDO",13
	.byt	"ENTIRE PATCH BY RESET SYS 64824.",13
	.byt	13
	.byt	"ASSEMBLY LANGUAGE PROGRAMS NEED TO BE",13
	.byt	"MODIFIED IF THEY USE LOAD OR SAVE.",13
	.byt  "ASM PROGRAMMERS:REFER TO DOCUMENTATION.",13
	.byt	0
init:

;;;;;;;;;;;;;;;;;
; check for ROM1
romresolve:			
	jsr	home
	lda	#32
	jsr	cout
	lda	screenxrom1
	pha
	lda	#BACKSPACE
	jsr	cout
	ldy	#<PROGRAMNAME
	jsr	print
	pla
	cmp	#1
	beq	@isrom1
@rom2or4:	
	ldy	#<NOTROM1
	jmp	print
@isrom1:
	lda	LDEND ;$F3CC known patch location, if #$20, rom is already patched for IEEE
	cmp	#$20
	bne	@overrom1patch
	ldy	#<ROM1PATCHED
	jmp	print
@overrom1patch:
;;;;;;;;;;;;;;;;;
; see if patch already applied
	lda	iC2+1 
	cmp	#<originalcode ;this check isn't necessary, and will change with each update
	bne	:+
	lda	iC2+2
	cmp	#>originalcode ;check 1 per dox
	bne	:+
	lda	iC2+3 
	cmp	#$2c ;check2 per dox
	bne	:+
	lda	tapebuffer2+8
	cmp	#$03 ;check 3 per fox
	bne	:+
	ldy	#<ALREADYPATCHED
	jsr	print
	lda	GOSAVE
	cmp	#$20
	beq	hiram_adjust_skip
	bne	hicodecopy
:

;;;;;;;;;;;;;;;;;	
; copy base code to final location
	ldx	#1+mycodeend-mycode
:
	lda	patchoffset-1,x
	sta	mycode-1,x
	dex
	bne	:-
;now copy/apply zero page patch
	ldx	#05
:
	lda	iC2,x
	sta	originalcode,x
	lda	zeropatch,x
	sta	iC2,x
	dex
	bpl	:-

	ldy	#<PATCHDETAIL
	jsr	printhigh

;;;;;;;;;;;;;;;;;
; copy hi code to appropriate place
hicodecopy:
hicode:
	lda	HIRAM
	sec
	sbc	#myhicodeend-myhicode
	sta	HIRAM
	bcs	@ovhi
	dec	HIRAM+1
@ovhi:
hiram_adjust_skip:
;do the actual copy
	ldy	#myhicodeend-myhicode-1
:
	lda	patchoffset+mycodeend-mycode,y

	sta	(HIRAM),y
	dey
	bpl	:-
	lda	#$20 ;jsr
	sta	GOSAVE
;now adjust the jump for JDOSAVEMACHINE and JDOSAVE
	lda	HIRAM
	;sta	JDOSAVE+1
	clc
	adc	#DOSAVEMACHINE-DOSAVE
	sta	JDOSAVEMACHINE+1
	lda	HIRAM+1
	;sta	JDOSAVE+2
	adc	#0
	sta	JDOSAVEMACHINE+2



	ldy	#<SAVEQUESTION
	jsr	printhigh
:
	jsr	getchar
	cmp	#'Y'
	beq	yes
	cmp	#'N'
	bne	:-
no:
	lda	#$60
	sta	GOSAVE
	clc
	lda	#myhicodeend-myhicode
	adc	HIRAM
	sta	HIRAM
	bne	:+
	inc	HIRAM+1
:
	ldy	#<SAVEDISABLED
	jsr	printhigh
	beq	done

yes:
	ldy	#<SAVEENABLED
	jsr	printhigh
done:
	ldy	#<RERUN
	jsr	printhigh

	jmp	CLR+2
home:
	lda	#147 ;CLR HOME
	jmp	cout
print:
	lda	#>MESSAGEBLOCK
	.byt	$2c ; bit operation to skip next LDA
printhigh:
	LDA	#(>MESSAGEBLOCK)+1
	sta	SELFMOD+2
	sty	SELFMOD+1
	ldy	#00
:
SELFMOD:
	lda	$2000,y
	beq	:+
	jsr	 cout
	iny
	bne	:-
	inc	SELFMOD+2
	bne	:-
:
	rts


zeropatch:
	;jsr	myinterpret
	jsr	originalcode
	.byt $2c ; bit operation to skip next two bytes
	.byt $00,$00 ; my zero page locations









patchoffset:
	.org	tapebuffer2
mycode:









LOADMACHINE:
	JSR	SETUPOUTSIDE
	BNE	DOLOADMACHINE
	JMP	SAVEMACHINE
GOSAVE:
	JSR	SETUP
	LDA   VARSTART ;$7C
	STA   EADDR
	LDA   VARSTART+1 ;$7D
	STA   EADDR+1
	; just fall through, OK to do slight repeat of setup to save bytes
SAVEMACHINE:
JDOSAVEMACHINE:
	JMP	DOSAVEMACHINE

originalcode:
	inc	Z
	bne   @ovhi
	inc	Z+1
@ovhi:

	ldy	#00
	lda	(Z),y
	cmp	#LOAD
	beq	DOLOAD
	cmp	#SAVE
	beq	GOSAVE

ovsave:
	CMP	#VERIFY
	bne	myrtn
	jsr	SETUP
	iny
	bne	MYLOAD

SETUP:
	LDA   #$04					; default load address to $0400
	STA   LADDR+1
	STY   LADDR ;Y=0
	;STY	VERFL	;Y=0
	JSR	iC2
	JSR	GETFPAR
SETUPOUTSIDE:
	LDA	FA
	CMP	#2
	BNE	:+
	JMP	DNPR				; Device not present error
:
	STA	MYZERO

SA60:
	LDA	#$60
	STA	SA
myrtn:

	rts
DOLOAD:
	JSR	SETUP


MYLOAD:
	STY	VERFL

IECLOAD:

	
keyloop:
	ldy	KEYMSK
	iny
	bne	keyloop
DOLOADMACHINE:
	LDA   FA						; check device address
	CMP	#$04
	BCS	IEEE488LOAD
	JMP	iF35A					; load entry point for devices less than 4

IEEE488LOAD:
	JSR   PRSRCH ;$F3FF		; print "searching for ..."
	JSR   IECOPEN ;$F462
	LDA	MYZERO
	STA	FA
	JSR   TALK
	LDA   SA
	JSR   SECTALK
	JSR   IECIN ;populate the load address lo/hi
	STA   EADDR
	JSR   IECIN
	STA   EADDR+1 ;18 bytes

	JSR   iF422					; print "loading/verifying"
iF37E:
	LSR	STATUS ;cheap way of clearing bit timeout
	JSR   IECABORT				;$F339 check stop key
	JSR   IECIN
	LDY   VERFL ;$020b
	BEQ   iF39A
	DEY
	CMP   (EADDR),Y				; compare when verifying
	BEQ   iF39C
	LDX   #$01
	STX   STATUS ;$020C
	BNE   DOLDEND ;doesn't match status set to 1, error
iF39A:
	STA   (EADDR),Y				; store when loading
iF39C:
	LDA	#2
	BIT	STATUS ;$020c
	BNE	iF37E ; loop back and try again

	INC   EADDR
	BNE   CKST
	INC   EADDR+1
CKST:
	BVC	iF37E ; leftover from bit operation, fall through if set
DOLDEND:
	LDA	#$5F ;setup for untalk
	JSR	iF182 ;untalk without saving FA
	JSR	CLSIEC
	JSR	MYUNLISTEN
	JMP	LDEND ;done loading
MYUNLISTEN:
	LDA	#$3F ;setup for unlisten
	JMP	iF182 ;unlisten without saving FA
mycodeend:
	.org	$1f00
myhicode:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
DOSAVE:
MYSAVE:
DOSAVEMACHINE:
	JSR	SETUPOUTSIDE

	LDA   FA
	CMP	#$04
	BCS	IEEE488SAVE
	JMP	iF6A1	; save entry point for devices less than 4
IEEE488SAVE:
	INC	SA ;SETUP sets to #$60, we need #61
	JSR   IECOPEN ;$F462
	LDA	MYZERO
	STA	FA
	JSR   LISTEN ;$F0BA
	LDA   SA
	JSR   SECOUT
	LDA   LADDR ;send the load address lo/hi
	JSR   IECOUT ;$F167
	LDA   LADDR+1
	JSR   IECOUT ;$F167 ;13 bytes
	LDY   #$00
	JSR   iFBDC					; LADR -> SADR


iF6CE:
	JSR   iFD90					; compare with end address
	BEQ   loopend
	LDA   (SADDR),Y
	JSR   IECOUT ;$F167
	JSR   IECABORT ;$F339
	INC   SADDR
	BNE   iF6CE
	INC   SADDR+1
	BNE   iF6CE
loopend:
	JSR   MYUNLISTEN ;$F17E
	JSR	CLSIEC
	JMP	MYUNLISTEN





myhicodeend:
.out	.sprintf("tb2 size: 0x%x %d", mycodeend-mycode, mycodeend-mycode);
.out	.sprintf("hi  size: 0x%x %d", myhicodeend-myhicode, myhicodeend-myhicode);
